@@ -10,9 +10,15 @@ import uuid
 from aiohttp import ClientSession
 from chip.clusters import Objects as Clusters
 
-from matter_server.common.errors import ERROR_MAP, NodeNotExists
+from matter_server.common.errors import ERROR_MAP, MatterError, NodeNotExists
 
-from ..common.helpers.util import dataclass_from_dict, dataclass_to_dict
+from ..common.helpers.util import (
+    convert_hex_string,
+    convert_mac_address,
+    create_attribute_path_from_attribute,
+    dataclass_from_dict,
+    dataclass_to_dict,
+)
 from ..common.models import (
     APICommand,
     CommandMessage,
@@ -22,6 +28,7 @@ from ..common.models import (
     MatterNodeData,
     MatterNodeEvent,
     MessageType,
+    NodePingResult,
     ResultMessageBase,
     ServerDiagnostics,
     ServerInfoMessage,
@@ -29,7 +36,13 @@ from ..common.models import (
 )
 from .connection import MatterClientConnection
 from .exceptions import ConnectionClosed, InvalidServerVersion, InvalidState
-from .models.node import MatterFabricData, MatterNode
+from .models.node import (
+    MatterFabricData,
+    MatterNode,
+    NetworkType,
+    NodeDiagnostics,
+    NodeType,
+)
 
 if TYPE_CHECKING:
     from chip.clusters.Objects import ClusterCommand
@@ -111,16 +124,28 @@ class MatterClient:
             return node
         raise NodeNotExists(f"Node {node_id} does not exist or is not yet interviewed")
 
-    async def commission_with_code(self, code: str) -> MatterNodeData:
+    async def commission_with_code(
+        self, code: str, network_only: bool = False
+    ) -> MatterNodeData:
         """
-        Commission a device using QRCode or ManualPairingCode.
+        Commission a device using a QR Code or Manual Pairing Code.
 
-        Returns basic MatterNodeData once complete.
+        :param code: The QR Code or Manual Pairing Code for device commissioning.
+        :param network_only: If True, restricts device discovery to network only.
+
+        :return: The NodeInfo of the commissioned device.
         """
-        data = await self.send_command(APICommand.COMMISSION_WITH_CODE, code=code)
+        data = await self.send_command(
+            APICommand.COMMISSION_WITH_CODE,
+            require_schema=6 if network_only else None,
+            code=code,
+            network_only=network_only,
+        )
         return dataclass_from_dict(MatterNodeData, data)
 
-    async def commission_on_network(self, setup_pin_code: int) -> MatterNodeData:
+    async def commission_on_network(
+        self, setup_pin_code: int, ip_addr: str | None = None
+    ) -> MatterNodeData:
         """
         Do the routine for OnNetworkCommissioning.
 
@@ -130,7 +155,10 @@ class MatterClient:
         Returns basic MatterNodeData once complete.
         """
         data = await self.send_command(
-            APICommand.COMMISSION_ON_NETWORK, setup_pin_code=setup_pin_code
+            APICommand.COMMISSION_ON_NETWORK,
+            require_schema=6 if ip_addr is not None else None,
+            setup_pin_code=setup_pin_code,
+            ip_addr=ip_addr,
         )
         return dataclass_from_dict(MatterNodeData, data)
 
@@ -177,6 +205,17 @@ class MatterClient:
         """
 
         node = self.get_node(node_id)
+        if node.available:
+            # try to refresh the OperationalCredentials.Fabric attribute
+            # so we have the most accurate information
+            attr_path = create_attribute_path_from_attribute(
+                0, Clusters.OperationalCredentials.Attributes.Fabrics
+            )
+            try:
+                await self.refresh_attribute(node_id, attr_path)
+            except MatterError as err:
+                self.logger.exception(err)
+
         fabrics: list[
             Clusters.OperationalCredentials.Structs.FabricDescriptorStruct
         ] = node.get_attribute_value(
@@ -210,6 +249,97 @@ class MatterClient:
             ),
         )
 
+    async def ping_node(self, node_id: int) -> NodePingResult:
+        """Ping node on the currently known IP-adress(es)."""
+        return cast(
+            NodePingResult,
+            await self.send_command(APICommand.PING_NODE, node_id=node_id),
+        )
+
+    async def node_diagnostics(self, node_id: int) -> NodeDiagnostics:
+        """Gather diagnostics for the given node."""
+        node = self.get_node(node_id)
+        # ping the node (will also refresh NetworkInterfaces data)
+        ping_result = await self.ping_node(node_id)
+        # grab some details from the first (operational) network interface
+        network_type = NetworkType.UNKNOWN
+        mac_address = None
+        attribute = Clusters.GeneralDiagnostics.Attributes.NetworkInterfaces
+        network_interface: Clusters.GeneralDiagnostics.Structs.NetworkInterface
+        for network_interface in node.get_attribute_value(
+            0, cluster=None, attribute=attribute
+        ):
+            # ignore invalid/non-operational interfaces
+            if not network_interface.isOperational:
+                continue
+            if (
+                network_interface.type
+                == Clusters.GeneralDiagnostics.Enums.InterfaceTypeEnum.kThread
+            ):
+                network_type = NetworkType.THREAD
+            elif (
+                network_interface.type
+                == Clusters.GeneralDiagnostics.Enums.InterfaceTypeEnum.kWiFi
+            ):
+                network_type = NetworkType.WIFI
+            elif (
+                network_interface.type
+                == Clusters.GeneralDiagnostics.Enums.InterfaceTypeEnum.kEthernet
+            ):
+                network_type = NetworkType.ETHERNET
+            else:
+                # unknown interface: ignore
+                continue
+            mac_address = convert_mac_address(network_interface.hardwareAddress)
+            break
+        # get thread/wifi specific info
+        node_type = NodeType.UNKNOWN
+        network_name = None
+        if network_type == NetworkType.THREAD:
+            cluster: Clusters.ThreadNetworkDiagnostics = node.get_cluster(
+                0, Clusters.ThreadNetworkDiagnostics
+            )
+            network_name = convert_hex_string(cluster.networkName)
+            # parse routing role to (diagnostics) node type
+            if (
+                cluster.routingRole
+                == Clusters.ThreadNetworkDiagnostics.Enums.RoutingRoleEnum.kSleepyEndDevice
+            ):
+                node_type = NodeType.SLEEPY_END_DEVICE
+            if cluster.routingRole in (
+                Clusters.ThreadNetworkDiagnostics.Enums.RoutingRoleEnum.kLeader,
+                Clusters.ThreadNetworkDiagnostics.Enums.RoutingRoleEnum.kRouter,
+            ):
+                node_type = NodeType.ROUTING_END_DEVICE
+            elif (
+                cluster.routingRole
+                == Clusters.ThreadNetworkDiagnostics.Enums.RoutingRoleEnum.kEndDevice
+            ):
+                node_type = NodeType.END_DEVICE
+        elif network_type == NetworkType.WIFI:
+            attr_value: bytes = node.get_attribute_value(
+                0,
+                cluster=None,
+                attribute=Clusters.WiFiNetworkDiagnostics.Attributes.Bssid,
+            )
+            network_name = convert_hex_string(attr_value)
+            node_type = NodeType.END_DEVICE
+        # override node type if node is a bridge
+        if node.node_data.is_bridge:
+            node_type = NodeType.BRIDGE
+        # get active fabrics for this node
+        active_fabrics = await self.get_matter_fabrics(node_id)
+        return NodeDiagnostics(
+            node_id=node_id,
+            network_type=network_type,
+            node_type=node_type,
+            network_name=network_name,
+            ip_adresses=list(ping_result),
+            mac_address=mac_address,
+            reachable=any(ping_result.values()),
+            active_fabrics=active_fabrics,
+        )
+
     async def send_device_command(
         self,
         node_id: int,
@@ -242,7 +372,7 @@ class MatterClient:
         node_id: int,
         attribute_path: str,
     ) -> Any:
-        """Read a single attribute on a node."""
+        """Read attribute(s) on a node."""
         return await self.send_command(
             APICommand.READ_ATTRIBUTE,
             require_schema=4,
@@ -250,6 +380,18 @@ class MatterClient:
             attribute_path=attribute_path,
         )
 
+    async def refresh_attribute(
+        self,
+        node_id: int,
+        attribute_path: str,
+    ) -> Any:
+        """Read attribute(s) on a node and store the updated value(s)."""
+        updated_values = await self.read_attribute(node_id, attribute_path)
+        if not isinstance(updated_values, dict):
+            updated_values = {attribute_path: updated_values}
+        for attr_path, value in updated_values.items():
+            self._nodes[node_id].update_attribute(attr_path, value)
+
     async def write_attribute(
         self,
         node_id: int,
